@@ -39,12 +39,14 @@ char*     mk_recurse_dir(char* dir, int32_t folder_type);
 int       close_recurse_dir();
 char*     mk_separate_dir(char *dir);
 int       close_separate_dir();
-int       mk_separate_file(struct file_ll *f, char *extension);
+void      mk_separate_file(struct file_ll *f, char *extension);
+void      close_separate_file(struct file_ll *f);
 char*     my_stristr(char *haystack, char *needle);
 void      check_filename(char *fname);
 void      write_separate_attachment(char f_name[], pst_item_attach* attach, int attach_num, pst_file* pst);
-void      write_embedded_message(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pf, char** extra_mime_headers);
+void      write_embedded_message(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pf, int save_rtf, char** extra_mime_headers);
 void      write_inline_attachment(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pst);
+int       valid_headers(char *header);
 void      header_has_field(char *header, char *field, int *flag);
 void      header_get_subfield(char *field, const char *subfield, char *body_subfield, size_t size_subfield);
 char*     header_get_field(char *header, char *field);
@@ -77,7 +79,7 @@ char*  kmail_chdir = NULL;
 #define MODE_KMAIL 1
 
 // recurse mode creates a directory structure like the PST file. Each directory
-// contains only one file which stores the emails in mbox format.
+// contains only one file which stores the emails in mboxrd format.
 #define MODE_RECURSE 2
 
 // separate mode creates the same directory structure as recurse. The emails are stored in
@@ -131,6 +133,7 @@ int         save_rtf_body = 1;
 int         file_name_len = 10;     // enough room for MODE_SPEARATE file name
 pst_file    pstfile;
 regex_t     meta_charset_pattern;
+char*       default_charset = NULL;
 
 int         number_processors = 1;  // number of cpus we have
 int         max_children  = 0;      // based on number of cpus and command line args
@@ -151,14 +154,27 @@ int grim_reaper(int waitall)
 #ifdef HAVE_FORK
 #ifdef HAVE_SEMAPHORE_H
     if (global_children) {
-        sem_getvalue(global_children, &available);
+        //sem_getvalue(global_children, &available);
         //printf("grim reaper %s for pid %d (parent %d) with %d children, %d available\n", (waitall) ? "all" : "", getpid(), getppid(), active_children, available);
         //fflush(stdout);
         int i,j;
         for (i=0; i<active_children; i++) {
+            int status;
             pid_t child = child_processes[i];
-            pid_t ch = waitpid(child, NULL, ((waitall) ? 0 : WNOHANG));
+            pid_t ch = waitpid(child, &status, ((waitall) ? 0 : WNOHANG));
             if (ch == child) {
+                // check termination status
+                //if (WIFEXITED(status)) {
+                //    int ext = WEXITSTATUS(status);
+                //    printf("Process %d exited with status  %d\n", child, ext);
+                //    fflush(stdout);
+                //}
+                if (WIFSIGNALED(status)) {
+                    int sig = WTERMSIG(status);
+                    DEBUG_INFO(("Process %d terminated with signal %d\n", child, sig));
+                    //printf("Process %d terminated with signal %d\n", child, sig);
+                    //fflush(stdout);
+                }
                 // this has terminated, remove it from the list
                 for (j=i; j<active_children-1; j++) {
                     child_processes[j] = child_processes[j+1];
@@ -300,6 +316,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                         pst_convert_utf8(item, &item->contact->address1);
                         fprintf(ff.output, "%s <%s>\n", item->contact->fullname.str, item->contact->address1.str);
                     }
+                    if (mode == MODE_SEPARATE) close_separate_file(&ff);
                 }
             }
 
@@ -318,8 +335,34 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                 else {
                     char *extra_mime_headers = NULL;
                     ff.item_count++;
-                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".eml" : "");
-                    write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                    if (mode == MODE_SEPARATE) {
+                        // process this single email message, possibly forking
+                        pid_t parent = getpid();
+                        pid_t child = try_fork(item->file_as.str);
+                        if (child == 0) {
+                            // we are the child process, or the original parent if no children were available
+                            pid_t me = getpid();
+                            mk_separate_file(&ff, (mode_EX) ? ".eml" : "");
+                            write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                            close_separate_file(&ff);
+#ifdef HAVE_FORK
+#ifdef HAVE_SEMAPHORE_H
+                            if (me != parent) {
+                                // we really were a child, forked for the sole purpose of processing this message
+                                // free my child count slot before really exiting, since
+                                // all I am doing here is waiting for my children to exit
+                                sem_post(global_children);
+                                grim_reaper(1); // wait for all my child processes to exit - there should not be any
+                                exit(0);        // really exit
+                            }
+#endif
+#endif
+                        }
+                    }
+                    else {
+                        // process this single email message, cannot fork since not separate mode
+                        write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                    }
                 }
             }
 
@@ -340,6 +383,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                     if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "");
                     write_journal(ff.output, item);
                     fprintf(ff.output, "\n");
+                    if (mode == MODE_SEPARATE) close_separate_file(&ff);
                 }
             }
 
@@ -360,6 +404,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                     if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "");
                     write_schedule_part_data(ff.output, item, NULL, NULL);
                     fprintf(ff.output, "\n");
+                    if (mode == MODE_SEPARATE) close_separate_file(&ff);
                 }
             }
 
@@ -399,11 +444,20 @@ int main(int argc, char* const* argv) {
     }
 
     // command-line option handling
-    while ((c = getopt(argc, argv, "bc:Dd:ehj:kMo:qrSt:uVw"))!= -1) {
+    while ((c = getopt(argc, argv, "bC:c:Dd:ehj:kMo:qrSt:uVw"))!= -1) {
         switch (c) {
         case 'b':
             save_rtf_body = 0;
             break;
+        case 'C':
+            if (optarg) {
+                default_charset = optarg;
+            }
+            else {
+                usage();
+                exit(0);
+            }
+            break;
         case 'c':
             if (optarg && optarg[0]=='v') {
                 contact_mode=CMODE_VCARD;
@@ -552,8 +606,7 @@ int main(int argc, char* const* argv) {
     DEBUG_ENT("main");
 
     if (output_mode != OUTPUT_QUIET) printf("Opening PST file and indexes...\n");
-
-    RET_DERROR(pst_open(&pstfile, fname), 1, ("Error opening File\n"));
+    RET_DERROR(pst_open(&pstfile, fname, default_charset), 1, ("Error opening File\n"));
     RET_DERROR(pst_load_index(&pstfile), 2, ("Index Error\n"));
 
     pst_load_extended_attributes(&pstfile);
@@ -617,13 +670,16 @@ int main(int argc, char* const* argv) {
 void write_email_body(FILE *f, char *body) {
     char *n = body;
     DEBUG_ENT("write_email_body");
-    while (n) {
-        if (strncmp(body, "From ", 5) == 0)
-            fprintf(f, ">");
-        if ((n = strchr(body, '\n'))) {
-            n++;
-            pst_fwrite(body, n-body, 1, f); //write just a line
-            body = n;
+    if (mode != MODE_SEPARATE) {
+        while (n) {
+            char *p = body;
+            while (*p == '>') p++;
+            if (strncmp(p, "From ", 5) == 0) fprintf(f, ">");
+            if ((n = strchr(body, '\n'))) {
+                n++;
+                pst_fwrite(body, n-body, 1, f); //write just a line
+                body = n;
+            }
         }
     }
     pst_fwrite(body, strlen(body), 1, f);
@@ -652,6 +708,7 @@ void usage() {
     printf("Usage: %s [OPTIONS] {PST FILENAME}\n", prog_name);
     printf("OPTIONS:\n");
     printf("\t-V\t- Version. Display program version\n");
+    printf("\t-C charset\t- character set for items with unspecified character set\n");
     printf("\t-D\t- Include deleted items in output\n");
     printf("\t-M\t- Write emails in the MH (rfc822) format\n");
     printf("\t-S\t- Separate. Write emails in the separate format\n");
@@ -683,9 +740,6 @@ void version() {
     printf("Little Endian implementation being used.\n");
 #else
 #  error "Byte order not supported by this library"
-#endif
-#ifdef __GNUC__
-    printf("GCC %d.%d : %s %s\n", __GNUC__, __GNUC_MINOR__, __DATE__, __TIME__);
 #endif
     DEBUG_RET();
 }
@@ -703,7 +757,7 @@ char *mk_kmail_dir(char *fname) {
         x = errno;
         DIE(("mk_kmail_dir: Cannot change to directory %s: %s\n", kmail_chdir, strerror(x)));
     }
-    dir = malloc(strlen(fname)+strlen(OUTPUT_KMAIL_DIR_TEMPLATE)+1);
+    dir = pst_malloc(strlen(fname)+strlen(OUTPUT_KMAIL_DIR_TEMPLATE)+1);
     sprintf(dir, OUTPUT_KMAIL_DIR_TEMPLATE, fname);
     check_filename(dir);
     if (D_MKDIR(dir)) {
@@ -712,17 +766,17 @@ char *mk_kmail_dir(char *fname) {
             DIE(("mk_kmail_dir: Cannot create directory %s: %s\n", dir, strerror(x)));
         }
     }
-    kmail_chdir = realloc(kmail_chdir, strlen(dir)+1);
+    kmail_chdir = pst_realloc(kmail_chdir, strlen(dir)+1);
     strcpy(kmail_chdir, dir);
     free (dir);
 
     //we should remove any existing indexes created by KMail, cause they might be different now
-    index = malloc(strlen(fname)+strlen(KMAIL_INDEX)+1);
+    index = pst_malloc(strlen(fname)+strlen(KMAIL_INDEX)+1);
     sprintf(index, KMAIL_INDEX, fname);
     unlink(index);
     free(index);
 
-    out_name = malloc(strlen(fname)+strlen(OUTPUT_TEMPLATE)+1);
+    out_name = pst_malloc(strlen(fname)+strlen(OUTPUT_TEMPLATE)+1);
     sprintf(out_name, OUTPUT_TEMPLATE, fname);
     DEBUG_RET();
     return out_name;
@@ -810,7 +864,7 @@ char *mk_separate_dir(char *dir) {
         if (y == 0)
             snprintf(dir_name, dirsize, "%s", dir);
         else
-      snprintf(dir_name, dirsize, "%s" SEP_MAIL_FILE_TEMPLATE, dir, y, ""); // enough for 9 digits allocated above
+            snprintf(dir_name, dirsize, "%s" SEP_MAIL_FILE_TEMPLATE, dir, y, ""); // enough for 9 digits allocated above
 
         check_filename(dir_name);
         DEBUG_INFO(("about to try creating %s\n", dir_name));
@@ -870,22 +924,34 @@ int close_separate_dir() {
 }
 
 
-int mk_separate_file(struct file_ll *f, char *extension) {
-    const int name_offset = 1;
+void mk_separate_file(struct file_ll *f, char *extension) {
     DEBUG_ENT("mk_separate_file");
     DEBUG_INFO(("opening next file to save email\n"));
     if (f->item_count > 999999999) { // bigger than nine 9's
         DIE(("mk_separate_file: The number of emails in this folder has become too high to handle\n"));
     }
-    sprintf(f->name, SEP_MAIL_FILE_TEMPLATE, f->item_count + name_offset, extension);
-    if (f->output) fclose(f->output);
-    f->output = NULL;
+    sprintf(f->name, SEP_MAIL_FILE_TEMPLATE, f->item_count, extension);
     check_filename(f->name);
     if (!(f->output = fopen(f->name, "w"))) {
         DIE(("mk_separate_file: Cannot open file to save email \"%s\"\n", f->name));
     }
     DEBUG_RET();
-    return 0;
+}
+
+
+void close_separate_file(struct file_ll *f) {
+    DEBUG_ENT("close_separate_file");
+    if (f->output) {
+        struct stat st;
+        fclose(f->output);
+        stat(f->name, &st);
+        if (!st.st_size) {
+            DEBUG_WARN(("removing empty output file %s\n", f->name));
+            remove(f->name);
+        }
+        f->output = NULL;
+    }
+    DEBUG_RET();
 }
 
 
@@ -940,6 +1006,7 @@ void write_separate_attachment(char f_name[], pst_item_attach* attach, int attac
     char *attach_filename = (attach->filename2.str) ? attach->filename2.str
                                                     : attach->filename1.str;
     DEBUG_ENT("write_separate_attachment");
+    DEBUG_INFO(("Attachment %s Size is %#"PRIx64", data = %#"PRIxPTR", id %#"PRIx64"\n", attach_filename, (uint64_t)attach->data.size, attach->data.data, attach->i_id));
 
     if (!attach->data.data) {
         // make sure we can fetch data from the id
@@ -982,7 +1049,7 @@ void write_separate_attachment(char f_name[], pst_item_attach* attach, int attac
 }
 
 
-void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pf, char** extra_mime_headers)
+void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pf, int save_rtf, char** extra_mime_headers)
 {
     pst_index_ll *ptr;
     DEBUG_ENT("write_embedded_message");
@@ -1010,9 +1077,13 @@ void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *bound
     if (!item) {
         DEBUG_WARN(("write_embedded_message: pst_parse_item was unable to parse the embedded message in attachment ID %llu", attach->i_id));
     } else {
-        fprintf(f_output, "\n--%s\n", boundary);
-        fprintf(f_output, "Content-Type: %s\n\n", attach->mimetype.str);
-        write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, 0, extra_mime_headers);
+        if (!item->email) {
+            DEBUG_WARN(("write_embedded_message: pst_parse_item returned type %d, not an email message", item->type));
+        } else {
+            fprintf(f_output, "\n--%s\n", boundary);
+            fprintf(f_output, "Content-Type: %s\n\n", attach->mimetype.str);
+            write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, save_rtf, extra_mime_headers);
+        }
         pst_freeItem(item);
     }
 
@@ -1022,9 +1093,8 @@ void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *bound
 
 void write_inline_attachment(FILE* f_output, pst_item_attach* attach, char *boundary, pst_file* pst)
 {
-    char *attach_filename;
     DEBUG_ENT("write_inline_attachment");
-    DEBUG_INFO(("Attachment Size is %"PRIu64", id %#"PRIx64"\n", (uint64_t)attach->data.size, attach->i_id));
+    DEBUG_INFO(("Attachment Size is %#"PRIx64", data = %#"PRIxPTR", id %#"PRIx64"\n", (uint64_t)attach->data.size, attach->data.data, attach->i_id));
 
     if (!attach->data.data) {
         // make sure we can fetch data from the id
@@ -1044,13 +1114,19 @@ void write_inline_attachment(FILE* f_output, pst_item_attach* attach, char *boun
     }
     fprintf(f_output, "Content-Transfer-Encoding: base64\n");
 
-    // If there is a long filename (filename2) use that, otherwise
-    // use the 8.3 filename (filename1)
-    attach_filename = (attach->filename2.str) ? attach->filename2.str : attach->filename1.str;
-    if (!attach_filename) {
+    if (attach->filename2.str) {
+        // use the long filename, converted to proper encoding if needed.
+        // it is already utf8
+        pst_rfc2231(&attach->filename2);
+        fprintf(f_output, "Content-Disposition: attachment; \n        filename*=%s\n\n", attach->filename2.str);
+    }
+    else if (attach->filename1.str) {
+        // short filename never needs encoding
+        fprintf(f_output, "Content-Disposition: attachment; filename=\"%s\"\n\n", attach->filename1.str);
+    }
+    else {
+        // no filename is inline
         fprintf(f_output, "Content-Disposition: inline\n\n");
-    } else {
-        fprintf(f_output, "Content-Disposition: attachment; filename=\"%s\"\n\n", attach_filename);
     }
 
     (void)pst_attach_to_file_base64(pst, attach, f_output);
@@ -1059,6 +1135,36 @@ void write_inline_attachment(FILE* f_output, pst_item_attach* attach, char *boun
 }
 
 
+int  valid_headers(char *header)
+{
+    // headers are sometimes really bogus - they seem to be fragments of the
+    // message body, so we only use them if they seem to be real rfc822 headers.
+    // this list is composed of ones that we have seen in real pst files.
+    // there are surely others. the problem is - given an arbitrary character
+    // string, is it a valid (or even reasonable) set of rfc822 headers?
+    if (header) {
+        if ((strncasecmp(header, "X-Barracuda-URL: ", 17) == 0) ||
+            (strncasecmp(header, "X-ASG-Debug-ID: ",  16) == 0) ||
+            (strncasecmp(header, "Return-Path: ",     13) == 0) ||
+            (strncasecmp(header, "Received: ",        10) == 0) ||
+            (strncasecmp(header, "Subject: ",          9) == 0) ||
+            (strncasecmp(header, "Date: ",             6) == 0) ||
+            (strncasecmp(header, "From: ",             6) == 0) ||
+            (strncasecmp(header, "X-x: ",              5) == 0) ||
+            (strncasecmp(header, "Microsoft Mail Internet Headers", 31) == 0)) {
+            return 1;
+        }
+        else {
+            if (strlen(header) > 2) {
+                DEBUG_INFO(("Ignore bogus headers = %s\n", header));
+            }
+            return 0;
+        }
+    }
+    else return 0;
+}
+
+
 void header_has_field(char *header, char *field, int *flag)
 {
     DEBUG_ENT("header_has_field");
@@ -1148,7 +1254,7 @@ int  test_base64(char *body)
     int b64 = 0;
     uint8_t *b = (uint8_t *)body;
     DEBUG_ENT("test_base64");
-    while (*b != 0) {
+    while (*b) {
         if ((*b < 32) && (*b != 9) && (*b != 10)) {
             DEBUG_INFO(("found base64 byte %d\n", (int)*b));
             DEBUG_HEXDUMPC(body, strlen(body), 0x10);
@@ -1279,7 +1385,13 @@ void write_schedule_part_data(FILE* f_output, pst_item* item, const char* sender
     fprintf(f_output, "PRODID:LibPST v%s\n", VERSION);
     if (method) fprintf(f_output, "METHOD:%s\n", method);
     fprintf(f_output, "BEGIN:VEVENT\n");
-    if (sender) fprintf(f_output, "ORGANIZER;CN=\"%s\":MAILTO:%s\n", item->email->outlook_sender_name.str, sender);
+    if (sender) {
+        if (item->email->outlook_sender_name.str) {
+            fprintf(f_output, "ORGANIZER;CN=\"%s\":MAILTO:%s\n", item->email->outlook_sender_name.str, sender);
+        } else {
+            fprintf(f_output, "ORGANIZER;CN=\"\":MAILTO:%s\n", sender);
+        }
+    }
     write_appointment(f_output, item);
     fprintf(f_output, "END:VCALENDAR\n");
 }
@@ -1327,7 +1439,9 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
     DEBUG_ENT("write_normal_email");
 
     pst_convert_utf8_null(item, &item->email->header);
-    headers = (item->email->header.str) ? item->email->header.str : *extra_mime_headers;
+    headers = valid_headers(item->email->header.str) ? item->email->header.str :
+              valid_headers(*extra_mime_headers)     ? *extra_mime_headers     :
+              NULL;
 
     // setup default body character set and report type
     strncpy(body_charset, pst_default_charset(item, sizeof(buffer_charset), buffer_charset), sizeof(body_charset));
@@ -1356,7 +1470,7 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
         else
             c_time = "Fri Dec 28 12:06:21 2001";
     } else
-        c_time= "Fri Dec 28 12:06:21 2001";
+        c_time = "Fri Dec 28 12:06:21 2001";
 
     // create our MIME boundaries here.
     snprintf(boundary, sizeof(boundary), "--boundary-LibPST-iamunique-%i_-_-", rand());
@@ -1373,7 +1487,8 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
             temp[1] = '\0';
             // pointer to all the embedded MIME headers.
             // we use these to find the actual rfc822 headers for embedded message/rfc822 mime parts
-            *extra_mime_headers = temp+2;
+            // but only for the outermost message
+            if (!*extra_mime_headers) *extra_mime_headers = temp+2;
             DEBUG_INFO(("Found extra mime headers\n%s\n", temp+2));
         }
 
@@ -1441,17 +1556,40 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
             fprintf(f_output, "%s", headers);
             // make sure the headers end with a \n
             if (headers[len-1] != '\n') fprintf(f_output, "\n");
+            //char *h = headers;
+            //while (*h) {
+            //    char *e = strchr(h, '\n');
+            //    int   d = 1;    // normally e points to trailing \n
+            //    if (!e) {
+            //        e = h + strlen(h);  // e points to trailing null
+            //        d = 0;
+            //    }
+            //    // we could do rfc2047 encoding here if needed
+            //    fprintf(f_output, "%.*s\n", (int)(e-h), h);
+            //    h = e + d;
+            //}
         }
     }
 
+    // record read status
+    if ((item->flags & PST_FLAG_READ) == PST_FLAG_READ) {
+        fprintf(f_output, "Status: RO\n");
+    }
+
     // create required header fields that are not already written
 
     if (!has_from) {
-        fprintf(f_output, "From: \"%s\" <%s>\n", item->email->outlook_sender_name.str, sender);
+        if (item->email->outlook_sender_name.str){
+            pst_rfc2047(item, &item->email->outlook_sender_name, 1);
+            fprintf(f_output, "From: %s <%s>\n", item->email->outlook_sender_name.str, sender);
+        } else {
+            fprintf(f_output, "From: <%s>\n", sender);
+        }
     }
 
     if (!has_subject) {
         if (item->subject.str) {
+            pst_rfc2047(item, &item->subject, 0);
             fprintf(f_output, "Subject: %s\n", item->subject.str);
         } else {
             fprintf(f_output, "Subject: \n");
@@ -1459,12 +1597,12 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
     }
 
     if (!has_to && item->email->sentto_address.str) {
-        pst_convert_utf8(item, &item->email->sentto_address);
+        pst_rfc2047(item, &item->email->sentto_address, 0);
         fprintf(f_output, "To: %s\n", item->email->sentto_address.str);
     }
 
     if (!has_cc && item->email->cc_address.str) {
-        pst_convert_utf8(item, &item->email->cc_address);
+        pst_rfc2047(item, &item->email->cc_address, 0);
         fprintf(f_output, "Cc: %s\n", item->email->cc_address.str);
     }
 
@@ -1595,7 +1733,7 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
                 attach->mimetype.str = strdup(RFC822);
                 attach->mimetype.is_utf8 = 1;
                 find_rfc822_headers(extra_mime_headers);
-                write_embedded_message(f_output, attach, boundary, pst, extra_mime_headers);
+                write_embedded_message(f_output, attach, boundary, pst, save_rtf, extra_mime_headers);
             }
             else if (attach->data.data || attach->i_id) {
                 if (mode == MODE_SEPARATE && !mode_MH)
@@ -1837,6 +1975,7 @@ void write_appointment(FILE* f_output, pst_item* item)
     pst_convert_utf8_null(item, &item->body);
     pst_convert_utf8_null(item, &appointment->location);
 
+    fprintf(f_output, "UID:%#"PRIx64"\n", item->block_id);
     fprintf(f_output, "DTSTAMP:%s\n",                     pst_rfc2445_datetime_format_now(sizeof(time_buffer), time_buffer));
     if (item->create_date)
         fprintf(f_output, "CREATED:%s\n",                 pst_rfc2445_datetime_format(item->create_date, sizeof(time_buffer), time_buffer));
@@ -1930,6 +2069,14 @@ void write_appointment(FILE* f_output, pst_item* item)
                 fprintf(f_output, "CATEGORIES:PHONE-CALL\n");
                 break;
         }
+        // ignore bogus alarms
+        if (appointment->alarm && (appointment->alarm_minutes >= 0) && (appointment->alarm_minutes < 1440)) {
+            fprintf(f_output, "BEGIN:VALARM\n");
+            fprintf(f_output, "TRIGGER:-PT%dM\n", appointment->alarm_minutes);
+            fprintf(f_output, "ACTION:DISPLAY\n");
+            fprintf(f_output, "DESCRIPTION:Reminder\n");
+            fprintf(f_output, "END:VALARM\n");
+        }
     }
     fprintf(f_output, "END:VEVENT\n");
     if (result) free(result);
@@ -2012,6 +2159,7 @@ void close_enter_dir(struct file_ll *f)
         pst_debug_unlock();
     }
     if (f->output) {
+        if (mode == MODE_SEPARATE) DEBUG_WARN(("close_enter_dir finds open separate file\n"));
         struct stat st;
         fclose(f->output);
         stat(f->name, &st);
@@ -2019,6 +2167,7 @@ void close_enter_dir(struct file_ll *f)
             DEBUG_WARN(("removing empty output file %s\n", f->name));
             remove(f->name);
         }
+        f->output = NULL;
     }
     free(f->name);
     free(f->dname);
